!!****J* QTLMap/toCarthagene
!!  NAME
!!   toCarthagene
!!  DESCRIPTION
!!   Build 2 types of dataset file for the carthagene binary from allele information deduced by QTLMap routines
!!     * Backcross format
!!     * Hybrid format
!!
!!   Format of the parameter file
!!     * in_genealogy=<filepath>
!!     * in_genotype=<filepath>
!!     * in_format=1|2  . 1 >intercross format, 2> hybrid format
!!     * opt_unknown_char=<code for missing data>
!!
!!   Notation Synonym      Possible Genotypes
!!
!!   1        A                 F0 |M0
!!   2                          F0 |M1
!!   3                       F0 |M0 , F0 |M1
!!   4                           F1 |M0
!!   5                       F0 |M0 , F1 |M0
!!   6        H              F0 |M1 , F1 |M0
!!   7        D          F0 |M0 , F0 |M1 , F1 |M0
!!   8        B                   F1 |M1
!!   9                       F0 |M0 , F1 |M1
!!   a                       F0 |M1 , F1 |M1
!!   b                  F0 |M0 , F0 |M1 , F1 |M1
!!   c                       F1 |M0 , F1 |M1
!!   d                  F0 |M0 , F1 |M0 , F1 |M1
!!   e                 F0 |M1 , F1 |M0 , F1 |M1
!!   f              F0 |M0 , F0 |M1 , F1 |M0 , F1 |M1
!!
!!   Backcross format result file
!!   ----------------------------
!!   data type f2 intercross
!!   <nb animal> <nbmarker> 0 0
!!   *<marker1> <1..f>...........
!!   *<marker2> <1..f>...........
!!
!!   Backcross format result file
!!   ----------------------------
!!   data type f2 backcross
!!   <nb animal> <nbmarker> 0 0
!!   *<marker1> <A,H,D,B>...........
!!   *<marker2> <A,H,D,B>...........
!!
!!
!!   *********
!!   Execution
!!   *********
!!   qtlmap2cartha p_analyse
!!
!!   an output file "to_carthagene_chr" will be created.
!!
!!
!!  BUGS
!!
!!  SEE ALSO
!!
!!    http://www.inra.fr/qtlmap
!!
!!  CREATION DATE
!!  28/08/2009
!!
!!  COPYRIGHT
!!***
program toCarthagene
      use m_qtlmap_log
      use m_qtlmap_constant
      use m_qtlmap_types
      use m_qtlmap_haplotype_util
      use m_qtlmap_genealogy
      use m_qtlmap_genotype
      use m_qtlmap_map
      use m_qtlmap_format_carthagene
      use m_qtlmap_output_handler

    implicit none

    character(len=LENGTH_MAX_FILE) :: outfile,param
    character(len=LEN_DEF)                  :: token
    integer :: stat,unit_genotype,nmarker_in_genotype_file,ios,type_out
    character(len=LEN_LINE*10)       :: line_read,temp
    logical :: is_ok
    integer :: nkeys = 0
    character(len=LEN_DEF) ,dimension(30) :: listKeys,listValues
    type(QTLMAP_DATASET)       :: dataset
    type(PDD_BUILD)            :: spt

    call dataset%set()

    if (dataset%cli%cli_is_help()) then
         print *," Generation of a maker file with transmission information from genotype and genealogy file (qtlmap format) "
         print *,""
         print *," The user have to define a p_analyse file with the following keys :"
         print *,"  * in_genealogy=<filepath>"
         print *,"  * in_genotype=<filepath>"
         print *,"  * in_format=1|2  . 1 >intercross format, 2> hybrid format"
         print *,"  * opt_unknown_char=<code for missing data>"
         print *,""
         stop
    end if


    ! ----- INIT LOG LEVEL
    call init_log(INFO_DEF)
    if (dataset%cli%cli_is_quiet()) call init_log(ERROR_DEF)
    if (dataset%cli%cli_is_verbose()) call init_log(VERBOSE_DEF)
    if (dataset%cli%cli_is_debug()) call init_log(DEBUG_DEF)

    ! ------ GETTING PARAMETER FILE
    param = dataset%cli%cli_get_parameters_file()
       !----- Give information about library used by the program
    call MATH_QTLMAP_INFO
    nkeys=0
    ! ------ READING PARAMETER FILE --> PARAMETER PART OF QTLMAP_DATA IS INITIALIZED
  !  call read_analyse_file(param,listKeys,listValues,30,nkeys)

    call initialize_tocartha_context(dataset,type_out)
    print *,"TYPE:",type_out
    unit_genotype = 200
    open (unit_genotype,FILE=dataset%params%get_file_val(K_INFORMATIVITY),IOSTAT=ios,recl=2**20)

    if (ios /= 0 ) then
      call stop_application("can not read typage file :"//trim(dataset%params%get_file_val(K_INFORMATIVITY)))
    end if

    line_read = ''

    do while ( trim(line_read) == '' .and. (ios == 0)  )
       call GET(unit_genotype,line_read,maxlen=LEN_LINE,IOSTAT=ios)
    end do

    if (ios /= 0) then
     call stop_application('No marker name are found in the genotype file. [file:'&
      //trim(dataset%params%get_file_val(K_INFORMATIVITY))//']')
    end if

    close(unit_genotype)

    nmarker_in_genotype_file = 0
    is_ok = .true.

    temp = line_read

    do while ( is_ok )
          token = trim (next_word(line_read,is_ok))
          if (is_ok) then
             nmarker_in_genotype_file = nmarker_in_genotype_file + 1
          end if
    end do

    call log_mess('number of marker in the header genotype file:'// trim(str(nmarker_in_genotype_file)) , INFO_DEF)

    if (nmarker_in_genotype_file == 0) then
         call stop_application('no marker found in ['//trim(dataset%params%get_file_val(K_INFORMATIVITY))//']')
    end if

    dataset%map%nmk(1) = nmarker_in_genotype_file
    allocate ( dataset%map%mark(dataset%map%nchr,nmarker_in_genotype_file))
    allocate ( dataset%map%posi(dataset%map%nchr,nmarker_in_genotype_file))
    allocate ( dataset%map%posif(dataset%map%nchr,nmarker_in_genotype_file))
    allocate ( dataset%map%posim(dataset%map%nchr,nmarker_in_genotype_file))
    allocate ( dataset%map%rm(dataset%map%nchr,nmarker_in_genotype_file,nmarker_in_genotype_file))
    allocate ( dataset%map%rf(dataset%map%nchr,nmarker_in_genotype_file,nmarker_in_genotype_file))

    line_read = temp
    is_ok = .true.
    nmarker_in_genotype_file = 0
    do while ( is_ok )
          token = trim (next_word(line_read,is_ok))
          if (is_ok) then
             nmarker_in_genotype_file = nmarker_in_genotype_file + 1
             dataset%map%mark(1,nmarker_in_genotype_file) = trim(token)
          end if
    end do

    !call read_map ! on lit une map...???
    call read_genealogy(dataset)
    call read_genotype(dataset)

    allocate (spt%prot(dataset%map%nchr,maxval(dataset%map%nmk),dataset%genea%nd,4),STAT=stat)
    call check_allocate(stat,'prot[toCarthagene]')
    call gammapf(dataset,spt)
    outfile = "to_carthagene_chr"
    call print_transmission_allele(dataset,spt%prot,outfile,type_out)
    deallocate (spt%prot)

    call dataset%release()

    contains

!!****f* m_qtlmap_parameter_file/initialize_tocartha_context
!!  NAME
!!    initialize_tocartha_context
!!  DESCRIPTION
!!    Initialize all public variables option come from the module m_qtlmap_data and file name to read for
!!   the binary qtlmap2cartha.
!!
!!  OUTPUTS
!!   type_out      :  intercross 1, hybrid 2
!!  NOTES
!!  SOURCE
      subroutine initialize_tocartha_context(dataset,type_out)
        type(QTLMAP_DATASET)       ,intent(inout)            :: dataset
        integer                    ,intent(out)              :: type_out ! format of file : intercross 1, hybrid 2
        character(len=LEN_W)   :: buf
        logical :: ok

        if ( .not. dataset%params%key_exist(K_GENEA) ) then
            call stop_application("Analysis case : You have to defined the key ["//trim(K_GENEA)//"]")
        end if

        if ( .not. dataset%params%key_exist(K_GENOTYPE) ) then
            call stop_application("Analysis case : You have to defined the key ["//trim(K_GENOTYPE)//"]")
        end if

        if ( .not. dataset%params%key_exist(K_UNKNOWN_GENO) ) then
          call stop_application("Analysis case : You have to defined the key ["//trim(K_UNKNOWN_GENO)//"]")
        end if

        dataset%map%nchr=1
        allocate (dataset%map%chromo(dataset%map%nchr))
        dataset%map%chromo='chromosome'
        allocate(dataset%map%nmk(dataset%map%nchr))
        call dataset%params%get_string_val(K_UNKNOWN_GENO,buf)
        call init_pheno(dataset%genoAnimal,dataset%map%nchr)
        buf=trim(buf)
        dataset%genoAnimal%nmanque =  set_pheno(dataset%genoAnimal,dataset%map,1,buf)

        if ( .not. dataset%params%key_exist("in_format") ) then
         call stop_application("Analysis case : You have to defined the key [in_format] 1 : intercross, 2 : hybrid")
        end if
        call dataset%params%get_string_val("in_format",buf)
        type_out = get_int(buf,ok)
        if ( .not. ok ) call stop_application("bad definition of key [in_format] expecting int value.")


      end subroutine  initialize_tocartha_context
!!***


end program toCarthagene
